@@ -137,6 +137,26 @@
 #define __GNUC_PREREQ(maj,min) 0
 #endif
 
+/**
+* AUGEAS_LIKELY:
+*
+* Macro to flag a code branch as a likely branch
+*
+* AUGEAS_UNLIKELY:
+*
+* Macro to flag a code branch as an unlikely branch
+*/
+#ifndef __has_builtin
+#   define __has_builtin(x) (0)
+#endif
+#if defined(__builtin_expect) || __has_builtin(__builtin_expect)
+#   define AUGEAS_LIKELY(x)        (__builtin_expect(!!(x), 1))
+#   define AUGEAS_UNLIKELY(x)      (__builtin_expect(!!(x), 0))
+#else
+#   define AUGEAS_LIKELY(x)        (x)
+#   define AUGEAS_UNLIKELY(x)      (x)
+#endif
+
 /**
  * ATTRIBUTE_UNUSED:
  *
@@ -168,6 +188,16 @@
 #endif
 #endif
 
+/* Allow falling through in switch statements for the few cases where that
+   is needed */
+#ifndef ATTRIBUTE_FALLTHROUGH
+#  if __GNUC_PREREQ (7, 0)
+#    define ATTRIBUTE_FALLTHROUGH __attribute__ ((fallthrough))
+#  else
+#    define ATTRIBUTE_FALLTHROUGH
+#  endif
+#endif
+
 /* A poor man's macro to get some move semantics: return the value of P but
    set P itself to NULL. This has the effect that if you say 'x = move(y)'
    that there is still only one pointer pointing to the memory Y pointed to
@@ -180,6 +210,7 @@
 #define ATTRIBUTE_FORMAT(...)
 #define ATTRIBUTE_PURE
 #define ATTRIBUTE_RETURN_CHECK
+#define ATTRIBUTE_FALLTHROUGH
 #define move(p) p
 #endif                                   /* __GNUC__ */
 
@@ -595,6 +626,12 @@ int pathx_symtab_assign_tree(struct pathx_symtab **symtab, const char *name,
 int pathx_symtab_undefine(struct pathx_symtab **symtab, const char *name);
 void pathx_symtab_remove_descendants(struct pathx_symtab *symtab,
                                      const struct tree *tree);
+/* Return the tree stored in the variable NAME at position I, which is the
+   same as evaluating the path expression '$NAME[I]'. If the variable NAME
+   does not exist, or does not contain a nodeset, or if I is bigger than
+   the size of the nodeset, return NULL */
+struct tree *pathx_symtab_get_tree(struct pathx_symtab *symtab,
+                                   const char *name, int i);
 void free_symtab(struct pathx_symtab *symtab);
 
 /* Escape a name so that it is safe to pass to parse_name and have it
