@@ -185,7 +185,7 @@ void free_skel(struct skel *skel);
 void free_dict(struct dict *dict);
 void free_lns_error(struct lns_error *err);
 
-/* Parse text TEXT with LENS. INFO indicats where TEXT was read from.
+/* Parse text TEXT with LENS. INFO indicates where TEXT was read from.
  *
  * If ERR is non-NULL, *ERR is set to NULL on success, and to an error
  * message on failure; the constructed tree is always returned. If ERR is
@@ -198,7 +198,17 @@ struct tree *lns_get(struct info *info, struct lens *lens, const char *text,
                      struct lns_error **err);
 struct skel *lns_parse(struct lens *lens, const char *text,
                        struct dict **dict, struct lns_error **err);
-void lns_put(FILE *out, struct lens *lens, struct tree *tree,
+
+/* Write tree TREE that was initially read from TEXT (but might have been
+ * modified) into file OUT using LENS.
+ *
+ * If ERR is non-NULL, *ERR is set to NULL on success, and to an error
+ * message on failure.
+ *
+ * INFO indicates where we are writing to, and its flags indicate whether
+ * to update spans or not.
+ */
+void lns_put(struct info *info, FILE *out, struct lens *lens, struct tree *tree,
              const char *text, struct lns_error **err);
 
 /* Free up temporary data structures, most importantly compiled
@@ -234,8 +244,14 @@ void free_lens(struct lens *lens);
 
    This range must include the ENC_* characters
 */
-#define RESERVED_FROM '\001'
-#define RESERVED_TO   ENC_SLASH_CH
+#define RESERVED_FROM "\001"
+#define RESERVED_TO   ENC_SLASH
+#define RESERVED_FROM_CH (RESERVED_FROM[0])
+#define RESERVED_TO_CH   ENC_SLASH_CH
+/* The range of reserved chars as it appears in a regex */
+#define RESERVED_RANGE_RX RESERVED_FROM "-" RESERVED_TO
+/* The equivalent of "." in a regexp for display */
+#define RESERVED_DOT_RX "[^" RESERVED_RANGE_RX "\n]"
 
 /* The length of the string S encoded */
 #define ENCLEN(s) ((s) == NULL ? strlen(ENC_NULL) : strlen(s))
