@@ -128,7 +128,7 @@ static const char *const axis_names[] = {
  *
  * See parse_name for the gory details
  */
-static const char const name_follow[] = "][|/=()!,";
+static const char name_follow[] = "][|/=()!,";
 
 /* Doubly linked list of location steps. Besides the information from the
  * path expression, also contains information to iterate over a node set,
@@ -295,11 +295,11 @@ static void func_regexp_flag(struct state *state, int nargs);
 static void func_glob(struct state *state, int nargs);
 static void func_int(struct state *state, int nargs);
 
-static const enum type const arg_types_nodeset[] = { T_NODESET };
-static const enum type const arg_types_string[] = { T_STRING };
-static const enum type const arg_types_bool[] = { T_BOOLEAN };
-static const enum type const arg_types_string_string[] = { T_STRING, T_STRING };
-static const enum type const arg_types_nodeset_string[] = { T_NODESET, T_STRING };
+static const enum type arg_types_nodeset[] = { T_NODESET };
+static const enum type arg_types_string[] = { T_STRING };
+static const enum type arg_types_bool[] = { T_BOOLEAN };
+static const enum type arg_types_string_string[] = { T_STRING, T_STRING };
+static const enum type arg_types_nodeset_string[] = { T_NODESET, T_STRING };
 
 static const struct func builtin_funcs[] = {
     { .name = "last", .arity = 0, .type = T_NUMBER, .arg_types = NULL,
@@ -1361,8 +1361,9 @@ static void eval_filter(struct expr *expr, struct state *state) {
     }
 }
 
-static struct value *lookup_var(const char *ident, struct state *state) {
-    list_for_each(tab, state->symtab) {
+static struct value *lookup_var(const char *ident,
+                                struct pathx_symtab *symtab) {
+    list_for_each(tab, symtab) {
         if (STREQ(ident, tab->name))
             return tab->value;
     }
@@ -1370,7 +1371,7 @@ static struct value *lookup_var(const char *ident, struct state *state) {
 }
 
 static void eval_var(struct expr *expr, struct state *state) {
-    struct value *v = lookup_var(expr->ident, state);
+    struct value *v = lookup_var(expr->ident, state->symtab);
     value_ind_t vind = clone_value(v, state);
     RET_ON_ERROR;
     push_value(vind, state);
@@ -1565,7 +1566,7 @@ static void check_binary(struct expr *expr, struct state *state) {
 }
 
 static void check_var(struct expr *expr, struct state *state) {
-    struct value *v = lookup_var(expr->ident, state);
+    struct value *v = lookup_var(expr->ident, state->symtab);
     if (v == NULL) {
         STATE_ERROR(state, PATHX_ENOVAR);
         return;
@@ -3017,6 +3018,19 @@ int pathx_symtab_assign_tree(struct pathx_symtab **symtab,
     return -1;
 }
 
+struct tree *
+pathx_symtab_get_tree(struct pathx_symtab *symtab,
+                      const char *name, int i) {
+    struct value *v = lookup_var(name, symtab);
+    if (v == NULL)
+        return NULL;
+    if (v->tag != T_NODESET)
+        return NULL;
+    if (i >= v->nodeset->used)
+        return NULL;
+    return v->nodeset->nodes[i];
+}
+
 void pathx_symtab_remove_descendants(struct pathx_symtab *symtab,
                                      const struct tree *tree) {
     list_for_each(tab, symtab) {
